/*
ChrispyChars Neck
	by Lach

v1
*/

if not CC_NeckSkins
	rawset(_G, "CC_NeckSkins", {})
end

--[[
USAGE: Copy the above into your script.

Make a CC_TrickSkins entry for each skin you want to support Neck.
The entry should be a table with as many of the following parameters as desired:
	- skin: Sets which skin should be used for this character's SPR2_HEDF, SPR2_HEDB, and SPR2_BODY sprites.
		If omitted, defaults to the skin being used in the CC_TrickSkins entry.
	- speed: The speed factor with which heads will turn to look around.
		FRACUNIT is instantaneous, 0 is no speed at all.
		If omitted, defaults to FRACUNIT/5.
	- pitches: The number of SPR2_HEDF/SPR2_HEDB frames this character has, for looking up and down.
		e.g. If your character has frames A, B, C, D, and E, set this to 5.
		If omitted, defaults to 1, which would mean only frame A is used.
		(The parameter superpitches can be used to change this value for a character's S_SUPER set.)
	- pitchinterval: An angle_t value that determines how far your camera must be tilted up/down before a change in pitch frame takes place.
		If omitted, defaults to ANGLE_22h.
		(The parameter superpitchinterval can be used to change this value for a character's S_SUPER set.)
	- pivot: A boolean that determines whether the character should use SPR2_BODY frames B and C while turning.
		If omitted, defaults to false.
		(The parameter superpivot can be used to change this value for a character's S_SUPER set.)
	- idleangle: An angle_t value that determines which angle the head should face relative to the camera when transitioning to their WAIT frames.
		If omitted, defaults to 0.
		(The parameter superidleangle can be used to change this value for a character's S_SUPER set.)
	- idlepitch: An angle_t value that determines which angle the head should look vertically when transitioning to their WAIT frames.
		For instance, Fluffy Tails, who looks up while yawning, has an idlepitch that angles the head such that frame B is usually displayed before the animation starts.
		If omitted, defaults to 0.
		(The parameter superidlepitch can be used to change this value for a character's S_SUPER set.)
	- searchradius: Determines how close objets have to be before the character can start to track them.
		If omitted, defaults to 512*FRACUNIT.
	- emoteoffset: A vertical offset for emote placement.
		For instance, Fluffy Tails uses a negative value so that he can share the same emotes as Pointy Sonic, but displayed slightly lower.
		If omitted, defaults to 0.
	- superfloatthreshold: A speed theshold, used for special behavior while super.
		If positive, the character will be allowed to move around while headtracking as long as their speed is less than this value.
		While moving or aeiral, the character will use the S_SUPER SPR2_BODY sprites.
		While still and grounded, the player will use the regular SPR2_BODY frames.
		If omitted, regular behavior is used for super forms.
	- eastereggname: If the player uses this name, the head will ignore the body angle when rotating. Just for fun!
	- trackables: See below.
	
See this wiki article for help with angle values: https://wiki.srb2.org/wiki/Constants#Angles_.28ANG.2A.2FANGLE_.2A.29
(You can mutltiply these by other numbers for even more angles, e.g. 2*ANG20 is 40 degress.)

While looking around, the character can track objects within their defined search radius. Any object without the MF_NOBLOCKMAP flag can be found.
For each object in range, a list of conditions is checked to determine whether it should be tracked.
The kinds of objects that can be tracked and with which priority is determined by the trackables parameter.
The trackables parameter is a table whose entries are ordered from most prioritized to least prioritized.
Each entry in the trackables table can have these parameters:
	- condition: function(item, player)
		The function should return true if player should track item.
		The function is only run if the object is alive and not behind a wall (at least as far as P_CheckSight can tell).
		(You can make exceptions to those conditions, see below.)
	- ignoresightcheck: If true, objects that meet the condition parameter can be tracked regardless of their visibility.
	- ignorehealthcheck: If true, objects that meet the condition parameter can be tracked whether they are alive or dead.
	- emoteskin: The skin the emote should use, should it need to differ from the player's.
		If omitted, defaults to the player's skin.
	- emotestate: The state the emote should be set to.
		If omitted, defaults to S_INVISIBLE, and therefore setting an emotesprite is recommended instead.
	- emotesprite: The sprite that the emote should use.
		If omitted, the sprite is not changed from when the emote is set to emotestate.
	- emotesprite2: If emotesprite is set to SPR_PLAY, this sprite2 will be used.
		If omitted, the sprite2 is not changed from when the emote is set to emotestate.
	- emoteframe: The frame the emote should use.
		Can be used to change the frame for the sprite set by emotesprite or emotestate.
		If omitted, the frame is not changed from when emotestate and/or emotesprite are applied.
	- emotecolor: The skincolor the emote should use.
		If omitted, defaults to SKINCOLOR_NONE.
	- emotesound: The sound the emote should play when the player starts viewing the tracked object.
		If omitted, no sound is played.
	- emotethink: function(emote, item, mo)
		Function that is called every frame so the emote can move or animate however desired.
		emote is the emote object, item is the object being tracked, and mo is the player object.
		Be mindful that the emote is teleported above the player each frame.
		If omitted, the emote remains still.
	- cusval: The emote object's cusval is set to this value before emotethink is called.
		Useful for making variations of an emotethink function within it.
		If omitted, the emote's cusval is set to 0.

See the other scripts in the ChrispyChars addon for some examples!

Include an **UNEDITED** copy of this script with your custom characters and have it load before their scripts.
You can also choose not to include the script, in which case your character's neck will only take effect when ChrispyChars or another CC_NECK-using character is added.
Please note that not all features of this script have been extensively tested, as they were only built with ChrispyChars in mind.
If you find bugs from using a parameter in a way I have not accounted for, please post a report in ChrispyChars' SRB2 Message Board Releases thread!
]]--

if CC_NECK
	return
end

rawset(_G, "CC_NECK", true)

local NF_LOOKING = 1 << 0
local NF_SKIPPEDFISRTLOOK = 1 << 1

rawset(_G, "CC_NECK_TRANSITIONTIME", 12)
local CC_NECK_SENSERANGE = 128*FRACUNIT

//local renderer = CV_FindVar("renderer")

local function valid(mo)
	return mo and mo.valid
end

rawset(_G, "CC_KillNeck", function(neck)
	neck.pivot = 0
	neck.anglesum = 0
	neck.flags = $ & ~NF_LOOKING | NF_SKIPPEDFISRTLOOK
	neck.target = nil
	for i = #neck.heads, 1, -1
		local head = neck.heads[i]
		if valid(head)
			P_RemoveMobj(head)
		end
		neck.heads[i] = nil
	end
end)

rawset(_G, "CC_RegisterNeckInfo", function(neckinfo, skin)
	if neckinfo.registered then return end
	
	if not neckinfo.skin
		neckinfo.skin = skin
	end
	
	if neckinfo.speed == nil
		neckinfo.speed = FRACUNIT/5
	end
	neckinfo.speed = min(max($, 0), FRACUNIT)
	
	if neckinfo.pitches == nil
	or neckinfo.pitches < 1
		neckinfo.pitches = 1
	end
	
	if neckinfo.superpitches == nil
	or neckinfo.superpitches < 1
		neckinfo.superpitches = neckinfo.pitches
	end
	
	if neckinfo.pitchinterval == nil
	or neckinfo.pitchinterval < ANG1
		neckinfo.pitchinterval = ANGLE_22h
	end
	
	if neckinfo.superpitchinterval == nil
	or neckinfo.superpitchinterval < ANG1
		neckinfo.superpitchinterval = neckinfo.pitchinterval
	end
	
	if neckinfo.superpivot == nil
		neckinfo.superpivot = neckinfo.pivot
	end
	
	if neckinfo.idleangle == nil
		neckinfo.idleangle = 0
	end
	
	if neckinfo.superidleangle == nil
		neckinfo.superidleangle = neckinfo.idleangle
	end
	
	if neckinfo.idlepitch == nil
		neckinfo.idlepitch = 0
	end
	
	if neckinfo.superidlepitch == nil
		neckinfo.superidlepitch = neckinfo.idlepitch
	end
	
	if neckinfo.searchradius == nil
		neckinfo.searchradius = RING_DIST
	end
	
	if neckinfo.emoteoffset == nil
		neckinfo.emoteoffset = 0
	end
	
	neckinfo.registered = true
end)

// To prevent duplicate freeslots
local function SafeFreeslot(...)
	for _, item in ipairs({...})
		if rawget(_G, item) == nil
			freeslot(item)
		end
	end
end

SafeFreeslot(
"SPR2_BODY",
"SPR2_HEDF",
"SPR2_HEDB",
"S_PLAY_LOOK",
"S_PLAY_HEAD",
"MT_HEAD_BACK",
"MT_HEAD_FRONT",
"MT_EMOTE")

function A_SetPAnim(mo, var1, var2)
	local player = mo.player
	if not player return end
	player.panim = var2
end

states[S_PLAY_LOOK] = {
	sprite = SPR_PLAY,
	frame = SPR2_BODY,
	tics = 6*TICRATE,
	action = A_SetPAnim,
	var2 = PA_IDLE,
	nextstate = S_PLAY_WAIT
}

states[S_PLAY_HEAD] = {
	sprite = SPR_PLAY,
	frame = SPR2_HEDF,
}

mobjinfo[MT_HEAD_BACK] = {
	spawnstate = S_PLAY_HEAD,
	height = mobjinfo[MT_PLAYER].height,
	radius = mobjinfo[MT_PLAYER].radius,
	flags = MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOGRAVITY|MF_SCENERY,
	dispoffset = -1,
}

mobjinfo[MT_HEAD_FRONT] = {
	spawnstate = S_PLAY_HEAD,
	height = mobjinfo[MT_PLAYER].height,
	radius = mobjinfo[MT_PLAYER].radius,
	flags = MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOGRAVITY|MF_SCENERY,
	dispoffset = 1,
}

mobjinfo[MT_EMOTE] = {
	spawnstate = S_INVISIBLE,
	reactiontime = 10,
	radius = 4*FRACUNIT,
	height = 8*FRACUNIT,
	flags = MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOGRAVITY|MF_SCENERY
}

local maxradius
local trackables
local bestobj
local bestdist
local maxcondition
local function LookForPOI(mo, obj, ignoredistancecheck)
	if not obj.valid
	or obj == mo
		return
	end
	
	local dist = FixedHypot(FixedHypot(mo.x - obj.x, mo.y - obj.y),
		mo.z + (mo.height >> 1) - (obj.z + (obj.height >> 1)))
		
	if dist > maxradius
	and not ignoredistancecheck
		return
	end
	
	local sightcheck = dist < FixedMul(CC_NECK_SENSERANGE, mo.scale) or P_CheckSight(mo, obj)
	
	for i = 1, maxcondition // check each condition
		if (sightcheck or trackables[i].ignoresightcheck)
		and (obj.health or trackables[i].ignorehealthcheck)
		and trackables[i].condition(obj, mo.player)
			
			// if i < maxcondition, we know this object is better than bestobj. Otherwise, find the closer of the two.
			if i == maxcondition and bestobj
				if dist > bestdist
					return
				end
			end
			
			// set bestobj
			bestobj = obj
			bestdist = dist
			maxcondition = i
			return
		end
	end
end

addHook("ThinkFrame", do
	for player in players.iterate
		local mo = player.mo
		local neckinfo
		local neck
		
		if not mo
			continue
		end
		
		neckinfo = CC_NeckSkins[mo.skin]
		
		if not (neckinfo and player.cc_neckenabled ~= false)
			if mo.neck
				if mo.state == S_PLAY_LOOK
					mo.state = S_PLAY_STND
				end
				for i = #mo.neck.heads, 1, -1
					local head = mo.neck.heads[i]
					if valid(head)
						head.state = S_INVISIBLE // "why not use P_RemoveMobj" well you see Fluffy Tails is full of hacks and removing his head crashes the game
						head.tics = 1
					end
				end
				mo.neck = nil
			end
			continue
		end
		
		// make sure neckinfo values are valid
		CC_RegisterNeckInfo(neckinfo, mo.skin)
		
		// setup
		if not mo.neck
			neck = {}
			neck.heads = {}
			neck.flags = 0
			neck.anglesum = 0
			neck.prevstate = S_PLAY_STND
			neck.super = player.powers[pw_super]
			neck.skin = mo.skin
			mo.neck = neck
		else
			neck = mo.neck
		end
		
		local grounded = P_IsObjectOnGround(mo)
		local superfloatthreshold = player.powers[pw_super] and neckinfo.superfloatthreshold
		
		// if the player is standing, they can start to look around
		if mo.state == S_PLAY_STND
		or superfloatthreshold and player.panim == PA_WALK and FixedDiv(player.speed, mo.scale) < superfloatthreshold
			mo.state = S_PLAY_LOOK
			neck.flags = $ | NF_LOOKING
			
			if superfloatthreshold
				player.panim = PA_DASH
			end
		end
		
		// directionchar hack
		if player.panim == PA_IDLE
			player.skidtime = max($, 1)
		end
		
		// if they're looking around, handle the heads
		if mo.state == S_PLAY_LOOK
			local floating = superfloatthreshold and (player.speed or not grounded) and not player.powers[pw_carry]
			local easteregg = neckinfo.eastereggname and player.name:lower() == neckinfo.eastereggname
			local legacy = player.pflags & (PF_DIRECTIONCHAR|PF_ANALOGMODE) ~= PF_DIRECTIONCHAR or G_RingSlingerGametype()
			
			// abort if there's a change in skin or super status
			if mo.skin ~= neck.skin
			or (player.powers[pw_super] and true) ~= (neck.super and true)
				neck.super = player.powers[pw_super]
				neck.skin = mo.skin
				mo.state = floating and S_PLAY_FLOAT or S_PLAY_STND
				CC_KillNeck(neck)
				continue
			end
			neck.super = player.powers[pw_super]
			neck.skin = mo.skin
			
			// if super, determine whether to use stand body or float body
			if superfloatthreshold
				if floating
					if FixedDiv(player.speed, mo.scale) >= superfloatthreshold
						mo.state = S_PLAY_WALK
						CC_KillNeck(neck)
						continue
					end
					mo.sprite2 = $ | FF_SPR2SUPER
					mo.tics = states[mo.state].tics
				else
					if superfloatthreshold
						mo.sprite2 = $ & ~FF_SPR2SUPER
					end
					player.skidtime = max($, 1)
				end
			end
			
			// spawn in head layers if they do not exist
			for i = 1, 2
				local head = neck.heads[i]
				if not valid(head)
					head = P_SpawnMobj(mo.x, mo.y, mo.z, MT_HEAD_BACK + i - 1)
					head.skin = neckinfo.skin
					head.state = $
					head.aiming = 0
					if i == 1
						head.sprite2 = SPR2_HEDB
						head.angle = player.drawangle
					end
					neck.heads[i] = head
				end
			end
			
			local head = neck.heads[1]
			local head2 = neck.heads[2]
			
			// look for a target, if the player should
			bestobj = nil
			trackables = neckinfo.trackables
			if trackables and player.cc_curiosityenabled ~= false and (neck.flags & NF_SKIPPEDFISRTLOOK or player.bot)
				maxcondition = #trackables
				maxradius = FixedMul(neckinfo.searchradius, mo.scale)
				
				// check current target - players can track them past the search radius unless a better object is found
				if neck.target
				and LookForPOI(mo, neck.target, player.powers[pw_carry] ~= CR_MINECART)
					bestobj = neck.target
				end
				
				// search for objects
				searchBlockmap("objects", LookForPOI, mo,
					mo.x - maxradius, mo.x + maxradius,
					mo.y - maxradius, mo.y + maxradius)
				
				if bestobj // we found something!
					// if an object is being tracked, the looking state should last longer
					if bestobj ~= neck.target // new target, let them look for a while
						mo.tics = max($, 2*TICRATE)
					elseif mo.tics >= CC_NECK_TRANSITIONTIME and leveltime & 1
						mo.tics = $ + 1
					end
				end
			end
			neck.target = bestobj
			
			// spawn an emotion, if the player should
			local emote = neck.emote
			if player.cc_emotionenabled ~= false
				if not valid(emote)
					emote = P_SpawnMobj(mo.x, mo.y, mo.z, MT_EMOTE)
					emote.tracer = mo
					neck.emote = emote
				end
				if bestobj
					emote.lastlook = maxcondition
				end
				emote.target = bestobj
			elseif valid(emote) // emotions are off, emote should fuck off
				neck.emote = nil
				emote.target = nil
			end
			
			// when true, the head angle is clamped to a 180° span in front of the player
			local lock
			if not bestobj
			or floating
				neck.anglesum = 0
			end
			if not easteregg
				lock = abs(neck.anglesum) > 360
				or legacy
				or floating
				or player.powers[pw_carry]
				or player.bot == 1 and players[0].mo
				and FixedHypot(players[0].mo.x - mo.x, players[0].mo.y - mo.y) < 32*mo.scale
			end
			
			// this is basically just for Pointy sneeze, but maybe it has other uses
			local headstate = (head.state - head.info.spawnstate or head2.state - head2.info.spawnstate) + head.info.spawnstate
			
			// angle head
			local angle
			local speed = neckinfo.speed
			
			if mo.tics >= 0 and mo.tics < CC_NECK_TRANSITIONTIME // we're getting bored. Move the head into WAIT position!
				if player.powers[pw_super]
					angle = player.drawangle + neckinfo.superidleangle
				else
					angle = player.drawangle + neckinfo.idleangle
				end
				lock = false
			else
				if bestobj // look at an object
					angle = R_PointToAngle2(mo.x, mo.y, bestobj.x, bestobj.y)
				elseif player.pflags & PF_ANALOGMODE
				or floating
					angle = player.drawangle
				else // look in the camera direction
					angle = mo.angle
					if legacy then speed = FRACUNIT end
				end
			end
			if lock // the body should not rotate, therefore the head angle needs to be clamped
				local diff = max(min(player.drawangle - angle, ANGLE_90 - 1), -ANGLE_90 + 1)
				angle = player.drawangle - diff
				
				diff = max(min(player.drawangle - head.angle, ANGLE_90 - 1), -ANGLE_90 + 1)
				head.angle = player.drawangle - diff
			end
			head.angle = $ + FixedMul(angle - $, speed)
			
			// aim head
			local aiming
			local span
			local halfspan
			local frame
			local pitches
			
			if headstate ~= head.info.spawnstate
				aiming = states[headstate].var1
				speed = FRACUNIT
				mo.tics = $ + 1
			elseif bestobj
				local objheight = (bestobj.height >> 1) + (bestobj.player and P_MobjFlip(bestobj)*(bestobj.height >> 2) or 0)
				aiming = R_PointToAngle2(
					0, mo.z + (mo.height >> 1) + P_MobjFlip(mo)*(mo.height >> 2),
					FixedHypot(mo.x - bestobj.x, mo.y - bestobj.y), bestobj.z + objheight)
			elseif player.powers[pw_carry]
				aiming = 0 // idk I guess minecarts don't update player.aiming
			else
				aiming = player.aiming
			end
			aiming = $*P_MobjFlip(mo)
			
			if player.powers[pw_super]
				if mo.tics >= 0 and mo.tics < CC_NECK_TRANSITIONTIME
					aiming = neckinfo.superidlepitch
				end
				pitches = neckinfo.superpitches
				span = max(ANGLE_90, neckinfo.superpitchinterval * pitches)
			else
				if mo.tics >= 0 and mo.tics < CC_NECK_TRANSITIONTIME
					aiming = neckinfo.idlepitch
				end
				pitches = neckinfo.pitches
				span = max(ANGLE_90, neckinfo.pitchinterval * pitches)
			end
			halfspan = span/2
			aiming = min(max($, -halfspan), halfspan)
			head.aiming = $ + FixedMul(aiming - $, speed)
			frame = pitches*(AngleFixed(head.aiming + halfspan) - 1)/AngleFixed(span)
			
			head.frame = frame | (mo.frame & ~FF_FRAMEMASK)
			
			// super?
			head.sprite2 = $ & ~FF_SPR2SUPER | (player.powers[pw_super] and FF_SPR2SUPER or 0)
			
			// copy data to other head layer
			head2.angle = head.angle
			head2.aiming = head.aiming
			head2.frame = head.frame
			head2.sprite2 = $ & ~FF_SPR2SUPER | (head.sprite2 & FF_SPR2SUPER)
			
			// copy player data to heads
			for i = 1, #neck.heads
				local h = neck.heads[i]
				h.scale = mo.scale
				h.height = mo.height
				h.flags2 = mo.flags2
				h.eflags = mo.eflags
				h.color = mo.color
				h.colorized = mo.colorized
				if h.skin ~= neckinfo.skin
					h.skin = neckinfo.skin
				end
				h.rollangle = mo.rollangle
				if headstate == head.info.spawnstate
					h.tics = 2
				end
				P_TeleportMove(h, mo.x, mo.y, mo.z)
			end
			
			// handle body pivoting
			local dopivot
			if mo.sprite2 & FF_SPR2SUPER
				dopivot = neckinfo.superpivot
			else
				dopivot = neckinfo.pivot
			end
			
			if not legacy // in Legacy mode the body doesn't rotate at all, so don't bother pivoting
			and not easteregg
				local bodyangle = player.drawangle
				local diff = head.angle - bodyangle
				if diff > ANGLE_90
					player.drawangle = head.angle - ANGLE_90
				elseif diff < -ANGLE_90
					player.drawangle = head.angle + ANGLE_90
				end
				diff = player.drawangle - bodyangle
				neck.anglesum = $ + diff/ANG1
				if diff > ANG1
					neck.pivot = -3
				elseif diff < -ANG1
					neck.pivot = 3
				end
			end
			
			frame = A
			if dopivot
			and neck.pivot
				if neck.pivot > 0
					frame = B
					neck.pivot = $ - 1
				else
					frame = C
					neck.pivot = $ + 1
				end
			end
			mo.frame = frame | ($ & ~FF_FRAMEMASK)
			
			// done!
			continue
		end
		
		// Not looking around? Remove the head objects
		if #neck.heads
			CC_KillNeck(neck)
		end
		
		neck.prevstate = mo.state
	end
end)

// Emote thinker

addHook("MobjThinker", function(emote)
	if not emote.valid return end
	
	local tracer = emote.tracer // tracer is the player
	if not (valid(tracer) and tracer.health and (CC_NeckSkins[tracer.skin] or emote.customskin))
		P_RemoveMobj(emote) // no player? you shouldn't exist!
		return
	end
	
	if emote.reactiontime // small delay so sounds don't instantly play upon standing still
		emote.reactiontime = $ - 1
		return
	end
	
	local player = tracer.player
	local neck = tracer.neck
	local neckinfo = CC_NeckSkins[tracer.skin]
	local data
	local target = emote.target // target is the object the player is looking at, if there is one
	
	if not neckinfo
		P_RemoveMobj(emote)
		return
	end
	
	CC_RegisterNeckInfo(neckinfo, tracer.skin)
	if emote.customskin
		data = CC_NeckSkins[emote.customskin]
		CC_RegisterNeckInfo(data, emote.customskin)
		data = $.trackables[emote.lastlook]
	else
		data = neckinfo.trackables[emote.lastlook]
	end
	
	if not data // edge case: if the player switches skins, the unupdated emotes' lastlook value may be greater than the number of entries in the new skin's trackables table
		return
	end
	
	if valid(target)
	and (neck and neck.flags & NF_LOOKING // while the player is looking at something, the emote will grow to its max size
		or emote.customskin)
	and data.emoteframe ~= -1
		emote.movecount = min($ + 1, 10)
	else
		emote.movecount = $ - 2
	end
	
	if emote.movecount <= 0 // shrunk to 0 size? no reason to exist anymore
		P_RemoveMobj(emote)
		return
	end
	
	emote.scale = emote.movecount*tracer.scale/10
	emote.eflags = tracer.eflags
	
	local x = tracer.x
	local y = tracer.y
	local z
	if emote.eflags & MFE_VERTICALFLIP
		z = tracer.z - emote.height - FixedMul(14*FRACUNIT + neckinfo.emoteoffset, tracer.scale)
	else
		z = tracer.z + FixedMul(skins[tracer.skin].height + 14*FRACUNIT + neckinfo.emoteoffset, tracer.scale)
	end
	if not splitscreen
		local camangle = R_PointToAngle(x, y)
		x = $ - FixedMul(cos(camangle), tracer.scale)
		y = $ - FixedMul(sin(camangle), tracer.scale)
	end
	P_TeleportMove(emote, x, y, z)
	
	if valid(target)
		local skin = data.emoteskin or emote.customskin or tracer.skin
		local state = data.emotestate or emote.info.spawnstate
		
		if emote.skin ~= skin
			emote.skin = skin
		end
		if emote.state ~= state
			emote.state = state
		end
		if data.emotesprite ~= nil
			emote.sprite = data.emotesprite
			if emote.sprite == SPR_PLAY
				emote.sprite2 = data.emotesprite2 or SPR2_STND
			end
		end
		if data.emoteframe ~= nil
			emote.frame = data.emoteframe
		end
		emote.color = data.emotecolor or SKINCOLOR_NONE
		emote.cusval = data.cusval or 0
		emote.rollangle = 0
		if emote.lastlook ~= emote.movedir
		and data.emotesound
			S_StartSound(emote, data.emotesound)
		end
		if data.emotethink
			data.emotethink(emote, target, tracer)
		end
		emote.movedir = emote.lastlook
		if emote.sprite == SPR_NULL
			emote.movecount = 0
		end
	end
end, MT_EMOTE)

// Commands

local function Boolean(arg)
	local num = tonumber(arg)
	if num ~= nil
		return num > 0
	end
	
	if arg == "on"
	or arg == "yes"
	or arg == "true"
		return true
	elseif arg == "off"
	or arg == "no"
	or arg == "false"
		return false
	end
end

local function GetEnabledString(val)
	return val and "enabled." or "disabled."
end

local function CommandToggle(player, arg, var, name, desc)
	local val
	if arg
		arg = $:lower()
		val = Boolean(arg)
		if val ~= nil
			player[var] = val
			if not player.cc_nosave
				CONS_Printf(splitscreen and consoleplayer or player, name.." "..GetEnabledString(player[var]))
				if CC_SaveConfiguration
					CC_SaveConfiguration(player)
				end
			end
			return
		end
	end
	
	CONS_Printf(splitscreen and consoleplayer or player, desc)
end

local function ToggleNeck(player, arg)
	local s = "cc_neckenabled"
	CommandToggle(player, arg, s, "Neck", "Enables/disables looking around with the head for supported custom characters.")
end

COM_AddCommand("neck2", ToggleNeck, 2)
COM_AddCommand("neck", ToggleNeck)

local function ToggleCuriosity(player, arg)
	local s = "cc_curiosityenabled"
	CommandToggle(player, arg, s, "Curiosity", "Enables/disables object tracking while Neck is enabled for supported custom characters.")
end

COM_AddCommand("curiosity2", ToggleCuriosity, 2)
COM_AddCommand("curiosity", ToggleCuriosity)

local function ToggleEmotion(player, arg)
	local s = "cc_emotionenabled"
	CommandToggle(player, arg, s, "Emotion", "Enables/disables emotions while Neck & Curiosity are enabled for supported custom characters.")
end

COM_AddCommand("emotion2", ToggleEmotion, 2)
COM_AddCommand("emotion", ToggleEmotion)